@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileAccess.cc,v 1.168 2011/03/25 14:35:41 lav Exp $ */
-
 #include <config.h>
 
 #include "FileAccess.h"
@@ -65,12 +63,12 @@ void FileAccess::Init()
    retries=0;
    opt_date=0;
    opt_size=0;
+   fileset_for_info=0;
    error_code=OK;
    saved_errno=0;
    mkdir_p=false;
    ascii=false;
    norest_manual=false;
-   array_ptr=array_cnt=0;
 
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
@@ -194,8 +192,8 @@ void FileAccess::Close()
    mode=CLOSED;
    opt_date=0;
    opt_size=0;
-   array_for_info=0;
-   array_ptr=array_cnt=0;
+   fileset_for_info=0;
+   retries=0;
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
    ascii=false;
@@ -287,7 +285,7 @@ const char *FileAccess::GetFileURL(const char *f,int flags) const
       if(!f || (f[0]!='/' && f[0]!='~'))
 	 f=dir_file(cwd.path?cwd.path.get():"~",f);
       u.path.set(f);
-      if(is_dir && url::dir_needs_trailing_slash(proto))
+      if(is_dir && url::dir_needs_trailing_slash(proto) && u.path.last_char()!='/')
 	 u.path.append('/');
    }
    return url.set_allocated(u.Combine(home));
@@ -365,12 +363,11 @@ void FileAccess::SetPasswordGlobal(const char *p)
    }
 }
 
-void FileAccess::GetInfoArray(struct fileinfo *info,int count)
+void FileAccess::GetInfoArray(FileSet *info)
 {
    Open(0,ARRAY_INFO);
-   array_for_info=info;
-   array_ptr=0;
-   array_cnt=count;
+   fileset_for_info=info;
+   fileset_for_info->rewind();
 }
 
 static void expand_tilde(xstring &path, const char *home, int i=0)
@@ -965,7 +962,7 @@ void FileAccess::Path::Change(const char *new_path,bool new_is_file,const char *
 	    new_url_path.set("/~");
       }
       if(new_url_path.last_char()!='/')
-	 new_url_path.append("/");
+	 new_url_path.append('/');
       if(new_path[0]=='/' || new_path[0]=='~' || new_device_prefix_len!=0)
       {
 	 bool have_slash=((new_path_enc?new_path_enc:new_path)[0]=='/');
