@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileAccess.h,v 1.116 2010/12/17 17:42:40 lav Exp $ */
-
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
 
@@ -69,16 +66,6 @@ public:
       SYMLINK,
    };
 
-   struct fileinfo
-   {
-      off_t size;
-      const char *file;
-      time_t time;
-      bool get_size:1;
-      bool get_time:1;
-      bool is_dir:1;
-   };
-
    class Path
    {
       void init();
@@ -135,13 +122,11 @@ protected:
    FileTimestamp *opt_date;
    off_t  *opt_size;
 
+   FileSet *fileset_for_info;
+
    time_t try_time;
    int retries;
 
-   fileinfo *array_for_info;
-   int	 array_ptr;
-   int	 array_cnt;
-
    bool	 mkdir_p;
 
    int	 saved_errno;
@@ -241,13 +226,8 @@ public:
    void RemoveDir(const char *dir)  { Open(dir,REMOVE_DIR); }
    void Chmod(const char *file,int m);
 
-   void	 GetInfoArray(struct fileinfo *info,int count);
-   int	 InfoArrayPercentDone()
-      {
-	 if(array_cnt==0)
-	    return 100;
-	 return array_ptr*100/array_cnt;
-      }
+   void	 GetInfoArray(FileSet *info);
+   int	 InfoArrayPercentDone() { return fileset_for_info->curr_pct(); }
 
    virtual const char *CurrentStatus();
 
@@ -424,7 +404,6 @@ protected:
 
 public:
    FileAccessRefC(const FileAccessRef& p) { ref=&p; }
-   ~FileAccessRefC() { close(); }
    const FileAccessRef& operator=(const FileAccessRef& p) { close(); ref=&p; return p; }
    operator const FileAccess*() const { return *ref; }
    FileAccess *operator->() const { return ref->operator->(); }
@@ -455,7 +434,7 @@ protected:
 
    bool use_cache;
 
-   ~FileAccessOperation() { if(session) session->Close(); }
+   void PrepareToDie() { if(session) session->Close(); }
 
 public:
    FileAccessOperation(FileAccess *s) : session(s), done(false), use_cache(true) {}
