@@ -955,7 +955,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    {
       if(!strcasecmp(value,"identity"))
 	 return;
-      if(!strcasecmp(value,"chunked"));
+      if(!strcasecmp(value,"chunked"))
       {
 	 chunked=true;
 	 chunk_size=CHUNK_SIZE_UNKNOWN;	  // expecting first chunk
