@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2014 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -628,7 +628,7 @@ Job *CmdExec::builtin_cd()
    int cache_is_dir=FileAccess::cache->IsDirectory(session,dir);
    if(cache_is_dir==1)
    {
-      if(is_file && dir_needs_slash)
+      if(is_file && dir_needs_slash && last_char(dir)!='/')
 	 dir=xstring::get_tmp(dir).append('/');
       is_file=false;
    }
@@ -2584,7 +2584,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2013);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2014);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -2850,13 +2850,15 @@ CMD(find)
    static struct option find_options[]=
    {
       {"maxdepth",required_argument,0,'d'},
+      {"ls",no_argument,0,'l'},
       {0,0,0,0}
    };
    int opt;
    int maxdepth = -1;
+   bool long_listing=false;
    const char *op=args->a0();
 
-   while((opt=args->getopt_long("+d:",find_options))!=EOF)
+   while((opt=args->getopt_long("+d:l",find_options))!=EOF)
    {
       switch(opt)
       {
@@ -2868,6 +2870,9 @@ CMD(find)
 	 }
 	 maxdepth = atoi(optarg);
 	 break;
+      case 'l':
+	 long_listing=true;
+	 break;
       case '?':
 	 eprintf(_("Usage: %s [-d #] dir\n"),op);
 	 return 0;
@@ -2878,6 +2883,7 @@ CMD(find)
       args->Append(".");
    FinderJob_List *j=new class FinderJob_List(session->Clone(),args.borrow(),output.borrow());
    j->set_maxdepth(maxdepth);
+   j->DoLongListing(long_listing);
    return j;
 }
 
