@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: complete.cc,v 1.84 2011/06/16 11:09:32 lav Exp $ */
-
 #include <config.h>
 
 #include "trio.h"
@@ -718,7 +716,6 @@ static char **lftp_completion (const char *text,int start,int end)
 	 glob_res->Add(f);
       }
       globfree(&pglob);
-      glob_res->ExcludeDots();
 
       rl_filename_completion_desired=1;
       generator = file_generator;
@@ -880,7 +877,7 @@ enum { COMPLETE_DQUOTE,COMPLETE_SQUOTE,COMPLETE_BSQUOTE };
 static char *
 single_quote (char *string)
 {
-  register int c;
+  int c;
   char *result, *r, *s;
 
   result = (char *)xmalloc (3 + (4 * strlen (string)));
@@ -909,7 +906,7 @@ single_quote (char *string)
 static char *
 double_quote (char *string)
 {
-  register int c;
+  int c;
   char *result, *r, *s;
 
   result = (char *)xmalloc (3 + (2 * strlen (string)));
