@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: xmap.cc,v 1.3 2011/03/10 07:00:05 lav Exp $ */
-
 #include <config.h>
 #include <assert.h>
 #include "xmap.h"
@@ -38,7 +36,7 @@ _xmap::_xmap(int vs)
    hash_size=1;
    new_map();
 }
-_xmap::~_xmap()
+void _xmap::_empty()
 {
    for(int i=0; i<hash_size; i++) {
       while(map[i])
@@ -46,6 +44,10 @@ _xmap::~_xmap()
    }
    assert(entry_count==0);
 }
+_xmap::~_xmap()
+{
+   _empty();
+}
 
 int _xmap::make_hash(const xstring& s) const
 {
@@ -135,7 +137,7 @@ void _xmap::_remove(entry **ep)
       return;
    entry *e=*ep;
    *ep=e->next;
-   delete e;
+   xfree(e);
    entry_count--;
 }
 
@@ -156,6 +158,7 @@ _xmap::entry *_xmap::_each_next()
       }
       each_entry=map[++each_hash];
    }
+   last_entry=0;
    return 0;
 }
 
